@@ -38,8 +38,6 @@ Todo:
 #define abs(x) _abs64(x)
 #endif
 
-#define ADM_ALLOWED_DRIFT_US 40000 // Allow 4b0 ms jitter on audio
-
 #if 1
 #define vprintf(...) {}
 #else
@@ -59,10 +57,28 @@ uint32_t        ADM_edAudioTrackFromVideo::getOutputFrequency(void)
         ADM_warning("No audio track\n");
         return 0;
     }
-    if(!trk->codec)
+    if(!trk->codec || trk->codec->configurationChanged())
         return trk->wavheader.frequency;
     return trk->codec->getOutputFrequency();
 }
+
+/**
+    \fn getOutputChannels
+    \brief get nb of channels as seen by codec
+*/
+uint32_t ADM_edAudioTrackFromVideo::getOutputChannels(void)
+{
+    ADM_audioStreamTrack *trk=getCurrentTrack();
+    if(!trk)
+    {
+        ADM_warning("No audio track\n");
+        return 0;
+    }
+    if(!trk->codec || trk->codec->configurationChanged())
+        return trk->wavheader.channels;
+    return trk->codec->getOutputChannels();
+}
+
 /**
     \fn     getPCMPacket
     \brief  Get audio packet
@@ -72,15 +88,25 @@ uint32_t        ADM_edAudioTrackFromVideo::getOutputFrequency(void)
 bool ADM_edAudioTrackFromVideo::getPCMPacket(float  *dest, uint32_t sizeMax, uint32_t *samples,uint64_t *odts)
 {
 uint32_t fillerSample=0;   // FIXME : Store & fix the DTS error correctly!!!!
-uint32_t inSize;
 bool drop=false;
+bool checkDts;
 static bool fail=false;
-uint32_t outFrequency=getCurrentTrack()->codec->getOutputFrequency();
+uint32_t outFrequency,outChannels;
+
  vprintf("[PCMPacket]  request TRK %d:%x\n",myTrackNumber,(long int)getCurrentTrack());
 again:
     *samples=0;
     ADM_audioStreamTrack *trk=getCurrentTrack();
     if(!trk) return false;
+    if(trk->codec->isDummy()) return false;
+
+    outChannels=getOutputChannels();
+    if(!outChannels) return false;
+
+    outFrequency=getOutputFrequency();
+    if(!outFrequency) return false;
+
+    checkDts=(trk->stream->constantSamplesPerPacket() && !trk->isbr);
     // Do we already have a packet ?
     if(!packetBufferSize)
     {
@@ -93,12 +119,12 @@ again:
         }
     }
     // We do now
-    vprintf("[PCMPacket]  TRK %d Got %d samples, time code %08lu  lastDts=%08lu delta =%08ld\n",
+    vprintf("[PCMPacket]  TRK %d Got %d samples, time code %08" PRIu64"  lastDts=%08" PRIu64" delta =%" PRId64"\n",
                 myTrackNumber,packetBufferSamples,packetBufferDts,lastDts,packetBufferDts-lastDts);
     fail=false;
 
     // Check if the Dts matches
-    if(lastDts!=ADM_AUDIO_NO_DTS &&packetBufferDts!=ADM_AUDIO_NO_DTS)
+    if(checkDts && lastDts!=ADM_AUDIO_NO_DTS &&packetBufferDts!=ADM_AUDIO_NO_DTS)
     {
         if(labs((int64_t)lastDts-(int64_t)packetBufferDts)>ADM_ALLOWED_DRIFT_US)
         {
@@ -120,7 +146,7 @@ again:
                 f/=1000000.;
                 // in samples!
                 uint32_t fillerSample=(uint32_t )(f+0.49);
-                uint32_t mx=sizeMax/trk->wavheader.channels;
+                uint32_t mx=sizeMax/outChannels;
                 
                 if(mx<fillerSample) fillerSample=mx;
                 // arbitrary cap, max 4kSample in one go
@@ -129,11 +155,11 @@ again:
                 {
                     fillerSample=4*1024;
                 }
-                uint32_t start=fillerSample*sizeof(float)*trk->wavheader.channels;
+                uint32_t start=fillerSample*sizeof(float)*outChannels;
                 memset(dest,0,start);
 
                 advanceDtsByCustomSample(fillerSample,outFrequency);
-                dest+=fillerSample*trk->wavheader.channels;
+                dest+=fillerSample*outChannels;
                 *samples=fillerSample;
                 vprintf("[Composer::getPCMPacket] Track %d:%x  Adding %u padding samples, dts is now %lu\n",
                             myTrackNumber,(long  int)trk,fillerSample,lastDts);
@@ -151,18 +177,17 @@ again:
     if(!trk->codec->run(packetBuffer, packetBufferSize, dest, &nbOut))
     {
             packetBufferSize=0; // consume
-            ADM_warning("[Composer::getPCMPacket] Track %d:%x : codec failed failed\n",
-                            myTrackNumber,trk);
+            ADM_warning("Track %d at 0x%p : decoding failed\n",myTrackNumber,trk);
             return false;
     }
     packetBufferSize=0; // consume
 
     // Compute how much decoded sample to compare with what demuxer said
     uint32_t decodedSample=nbOut;
-    decodedSample/=trk->wavheader.channels;
+    decodedSample/=outChannels;
     if(!decodedSample) goto again;
 #define ADM_MAX_JITTER 5000  // in samples, due to clock accuracy, it can be +er, -er, + er, -er etc etc
-    if(labs((int64_t)decodedSample-(int64_t)packetBufferSamples)>ADM_MAX_JITTER)
+    if(checkDts && labs((int64_t)decodedSample-(int64_t)packetBufferSamples)>ADM_MAX_JITTER)
     {
         ADM_warning("[Composer::getPCMPacket] Track %d:%x Demuxer was wrong %d vs %d samples!\n",
                     myTrackNumber,trk,packetBufferSamples,decodedSample);
@@ -178,9 +203,9 @@ again:
     *samples=(decodedSample);
     *odts=lastDts;
     advanceDtsByCustomSample(decodedSample,outFrequency);
-    vprintf("[Composer::getPCMPacket] Track %d:%x Adding %u decoded, Adding %u filler sample, dts is now %lu\n",
-                    myTrackNumber,(long int)trk,  decodedSample,fillerSample,lastDts);
-    ADM_assert(sizeMax>=(fillerSample+decodedSample)*trk->wavheader.channels);
+    vprintf("[Composer::getPCMPacket] Track %d:%p Adding %u decoded, Adding %u filler sample, dts is now %" PRIu64", fq: %d\n",
+                    myTrackNumber,trk,decodedSample,fillerSample,lastDts,outFrequency);
+    ADM_assert(sizeMax>=(fillerSample+decodedSample)*outChannels);
     return true;
 }
 
