@@ -52,6 +52,8 @@
 #include "ADM_threads.h"
 #include "ADM_muxerProto.h"
 
+#include "ADM_filterChain.h"
+
 static admMutex singleThread("actionHandlerMutex");
 static int cutsNotOnIntraWarned;
 
@@ -68,6 +70,7 @@ extern const char * GUI_getCustomPyScript(uint32_t nb);
 extern const char * GUI_getAutoPyScript(uint32_t nb);
 extern void call_scriptEngine(const char *scriptFile);
 extern int     GUI_handleVFilter (void);
+extern int     GUI_handleVPartialFilter (void);
 // Debug functions
        void    GUI_showCurrentFrameHex(void);
        void    GUI_showSize(void);
@@ -313,7 +316,21 @@ void HandleAction (Action action)
             EditableAudioTrack *ed=video_body->getDefaultEditableAudioTrack();
             if(ed)
             {
-                ed->audioEncodingConfig.audioFilterConfigure();
+                double tempoHint = 1.0;
+                if(UI_getCurrentVCodec())
+                {
+                    ADM_videoFilterChain *videoChain=createVideoFilterChain(0,ADM_NO_PTS);
+                    ADM_coreVideoFilter * videoFilter=(*videoChain)[0];
+                    FilterInfo *info=videoFilter->getInfo();
+                    tempoHint = info->totalDuration;
+                    int nb=videoChain->size();
+                    videoFilter=(*videoChain)[nb-1];
+                    info=videoFilter->getInfo();
+                    tempoHint /= info->totalDuration;
+                    destroyVideoFilterChain(videoChain);
+                    //printf("[audioFilterConfigure] tempo hint: %f\n",tempoHint);
+                }
+                ed->audioEncodingConfig.audioFilterConfigure(tempoHint);
                 UI_setTimeShift(ed->audioEncodingConfig.shiftEnabled,ed->audioEncodingConfig.shiftInMs);
             }
         }
@@ -487,9 +504,6 @@ void HandleAction (Action action)
       if (playing)
     GUI_PlayAvi ();
       break;
-    case ACT_SetPostProcessing:
-      A_setPostproc();
-      break;
     case ACT_MarkA:
     case ACT_MarkB:
     {
@@ -729,7 +743,7 @@ void HandleAction (Action action)
                 a=b;
                 b=p;
             }
-            // Special case of B being at or beyond the last frame
+            // Special case of B being beyond the last frame
             bool lastFrame=false;
             if(b==before)
             {
@@ -741,7 +755,7 @@ void HandleAction (Action action)
                 {
                     GUI_infiniteForward(pts);
                     uint64_t lastFramePts=video_body->getCurrentFramePts();
-                    if(b>=lastFramePts) lastFrame=true; // B is at or beyond the last frame
+                    if(b > lastFramePts) lastFrame=true; // B is beyond the last frame
                 }
             }
             if(!a && lastFrame)
@@ -887,6 +901,9 @@ void HandleAction (Action action)
     case ACT_VIDEO_FILTERS:
         GUI_handleVFilter();
         break;
+    case ACT_VIDEO_PARTIAL_FILTERS:
+        GUI_handleVPartialFilter();
+        break;
 
    case ACT_HEX_DUMP:
       GUI_showCurrentFrameHex();
@@ -1476,8 +1493,9 @@ extern int DIA_getMPParams( uint32_t *pplevel, uint32_t *ppstrength,bool *swap);
 //
 void    A_setPostproc( void )
 {
-uint32_t type,strength;
-bool swap;
+    uint32_t type,strength;
+    bool swap;
+    stagedActionSuccess = 0;
     if(!avifileinfo) return;
 
     video_body->getPostProc(&type,&strength,&swap);
@@ -1485,25 +1503,34 @@ bool swap;
      if(DIA_getMPParams( &type, &strength,&swap))
      {
         video_body->setPostProc(type,strength,swap);
+        stagedActionSuccess = 1;
+        return;
      }
 
 }
 
 //
-extern int DIA_getHDRParams( uint32_t * toneMappingMethod, float * saturationAdjust, float * boostAdjust);
+extern int DIA_getHDRParams( uint32_t * toneMappingMethod, float * saturationAdjust, float * boostAdjust, bool * adaptiveRGB, uint32_t * gamutMethod);
 //
 void    A_setHDRConfig( void )
 {
-    uint32_t method;
+    uint32_t method,gamutMethod;
     float saturation, boost;
+    bool adaptiveRGB;
     stagedActionSuccess = 0;
     if(!avifileinfo) return;
 
-    video_body->getHDRConfig(&method,&saturation,&boost);
+    if(!video_body->getHDRConfig(&method,&saturation,&boost,&adaptiveRGB,&gamutMethod))
+    {
+        method = 1;
+        saturation = boost = 1.;
+        adaptiveRGB = true;
+        gamutMethod = 0;
+    }
 
-     if(DIA_getHDRParams( &method, &saturation,&boost))
+     if(DIA_getHDRParams( &method, &saturation,&boost,&adaptiveRGB,&gamutMethod))
      {
-        video_body->setHDRConfig(method,saturation,boost);
+        video_body->setHDRConfig(method,saturation,boost,adaptiveRGB,gamutMethod);
         stagedActionSuccess = 1;
         return;
      }
