@@ -40,6 +40,7 @@ static void A_timedError(bool *first, const char *s);
 
 extern uint8_t DIA_gotoTime(uint32_t *hh, uint32_t *mm, uint32_t *ss,uint32_t *ms);
 extern void A_setHDRConfig(void);
+extern void A_setPostproc(void);
 
 static uint32_t jumpTarget[4] = {0};
 
@@ -61,6 +62,9 @@ void HandleAction_Staged(Action action)
         case ACT_SetHDRConfig:
             A_setHDRConfig();
             break;
+        case ACT_SetPostProcessing:
+            A_setPostproc();
+            break;
         case ACT_SelectTime:
             {
                 stagedActionSuccess = 0;
@@ -591,8 +595,10 @@ bool A_jumpToTime(uint32_t hh,uint32_t mm,uint32_t ss,uint32_t ms)
     pts*=1000;
     pts+=ms;
     pts*=1000;
+    // Aim higher to avoid rejecting the target picture if PTS is not a multiple of 1000 us.
+    // We can overshoot if two pics are less than 1ms apart, an unlikely scenario.
+    pts+=999;
 
-    pts++;
     if(pts >= total)
         pts = total-1;
     if(false==GUI_lastFrameBeforePts(pts)) // we are probably at the beginning of the video,
@@ -864,9 +870,13 @@ bool GUI_infiniteForward(uint64_t pts)
             return false;
         }
     }
+    bool refreshNeeded = true;
     while(admPreview::nextPicture())
     {
+        refreshNeeded = false;
     }
+    if(refreshNeeded)
+        admPreview::samePicture();
     admPreview::deferDisplay(0);
     return true;
 }
@@ -906,7 +916,7 @@ bool GUI_lastFrameBeforePts(uint64_t pts)
     admPreview::deferDisplay(false);
     admPreview::samePicture();
     GUI_setCurrentFrameAndTime();
-
+    UI_purge();
     return true;
 }
 
