@@ -4,30 +4,41 @@
 #ifndef Q_SHELL_H
 #define Q_SHELL_H
 
-#include <QItemDelegate>
 #include "ui_shell.h"
 
 #include "ADM_inttype.h"
 #include "IScriptEngine.h"
 
+#include <vector>
+
+typedef struct
+{
+    std::string name;
+    QString *command;
+} shellHistoryEntry;
+
 /**
     \class ADM_jsQt4Shell
 */
-#define Q_SHELL_HISTORY 8 // Must be a power of 2!
 class qShell: public QDialog
 {
 	Q_OBJECT
+#ifdef SCRIPT_SHELL_HISTORY_VERBOSE
+    bool                 dumpHistory(void);
+#endif
+    void                 showEvent(QShowEvent *ev);
 protected:
     IScriptEngine      *_engine;
     Ui_SpiderMonkeyShell ui;
     bool                 eventFilter(QObject* watched, QEvent* event);
     bool                 previousCommand(void);
     bool                 nextCommand(void);
-    QString              *history[Q_SHELL_HISTORY];
+    std::vector<shellHistoryEntry> *_history;
+    int                  addToHistory(QString &str);
     int                  indexWrite;
     int                  indexRead;
 public:
-                    qShell(QWidget *parent, IScriptEngine *engine);
+                    qShell(QWidget *parent, IScriptEngine *engine, std::vector <shellHistoryEntry> *commands);
     virtual         ~qShell() ;
     bool            run(void);
     bool            print(IScriptEngine::EventType type, const char *s);
