@@ -26,7 +26,7 @@ decoderRGB16::decoderRGB16(uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDa
     : decoders (  w,   h,  fcc,   extraDataLen,   extraData,  bpp)
 {
     _bpp = bpp;
-    bytePerPixel=_bpp>>3;
+    bytePerPixel=(_bpp & 0x3f)>>3;
     decoded = new uint8_t[2*bytePerPixel * w * h];
 }
 /**
@@ -43,7 +43,7 @@ decoderRGB16::~decoderRGB16()
 bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
 {
         int lineSize = (_w *bytePerPixel + 3) & ~3; // 4 bytes aligned ?
-        ADM_colorspace colorspace;
+        ADM_pixelFormat pixFrmt;
         int i, j;
         uint8_t *src = in->data;
         uint8_t *dst = decoded;
@@ -53,11 +53,12 @@ bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
         {
                 case 16:
                         // FIXME - 16-bit could use a BGR555 or BGR565 colour mask
-                        colorspace = ADM_COLOR_BGR555;
+                        pixFrmt = ADM_PIXFRMT_RGB555;
                         break;
                 case 24:
-                case 32:
-                        colorspace = ADM_COLOR_RGB24;
+                case 32: // BGRx
+                case 96: // fake bpp indicating xBGR
+                        pixFrmt = ADM_PIXFRMT_BGR24;
                         break;
                 default:
                         printf("bpp %d not supported\n", _bpp);
@@ -66,7 +67,7 @@ bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
     // Pack...
         // Invert scanline
         src = in->data+lineSize*(_h-1);
-        if (_bpp == 32) // 32 -> 24
+        if (_bpp == 32 || _bpp == 96) // 32 -> 24
         {
             outBytePerPixel=3;   
             destStride=outBytePerPixel*_w;
@@ -77,11 +78,19 @@ bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
 
                     for(j = 0; j < _w; j++) // 32 32 32 32
                     {
-                            ptr[0] = buf[1]; // remove alpha channel + reorder. IT would be more efficient to do it in colorspace...
-                            ptr[1] = buf[0];
+                        if(_bpp == 32)
+                        {
+                            ptr[0] = buf[0]; // remove alpha channel + reorder. IT would be more efficient to do it in colorspace...
+                            ptr[1] = buf[1];
                             ptr[2] = buf[2];
-                            ptr += 3;
-                            buf += 4;
+                        }else // xBGR
+                        {
+                            ptr[0] = buf[1];
+                            ptr[1] = buf[2];
+                            ptr[2] = buf[3];
+                        }
+                        ptr += 3;
+                        buf += 4;
                     }
                     src -= lineSize;
                     dst += _w * 3;                    
@@ -104,7 +113,8 @@ bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
         ADM_assert(out->isRef());
         ADMImageRef *ref=out->castToRef();
         out->flags = AVI_KEY_FRAME;
-        out->_colorspace = colorspace;
+        out->_pixfrmt = pixFrmt;
+        out->_range = ADM_COL_RANGE_JPEG; // full
 
         ref->_planes[0] = decoded;
         ref->_planes[1] = NULL;
@@ -114,6 +124,13 @@ bool decoderRGB16::uncompress(ADMCompressedImage * in, ADMImage * out)
         ref->_planeStride[1] = 0;
         ref->_planeStride[2] = 0;
         out->Pts=in->demuxerPts;
+        // make sure the output is not marked as a hw image
+        int count = 0;
+        while(out->refType != ADM_HW_NONE && count < 32 /* arbitrary limit */)
+        {
+            out->hwDecRefCount();
+            count++;
+        }
         return true;
 }
 //EOF
