@@ -49,6 +49,8 @@ x264Encoder::x264Encoder(ADM_coreVideoFilter *src,bool globalHeader) : ADM_coreV
     this->globalHeader=globalHeader;
     passNumber=0;
     logFile=NULL;
+    flush=false;
+    firstIdr=true;
 }
 
 /**
@@ -162,56 +164,63 @@ bool         x264Encoder::encode (ADMBitstream * out)
 {
     // 1 fetch a frame...
     uint32_t nb;
+    x264_picture_t pic_out;
+    x264_nal_t *nal;
+    int er,nbNal;
 
     // update
-again:    
-    bool gotFrame=true;
-    if(source->getNextFrame(&nb,image)==false)
+again:
+    if(!flush)
     {
-        ADM_warning("[x264] Cannot get next image\n");
-        gotFrame=false;
-        //return false;
+        if(source->getNextFrame(&nb,image))
+        {
+            if(image->_range == ADM_COL_RANGE_JPEG && !param.vui.b_fullrange)
+                image->shrinkColorRange();
+            // 2-preamble
+            if(false==preAmble(image))
+            {
+                ADM_warning("[x264] preAmble failed\n");
+                return false;
+            }
+        }else
+        {
+            ADM_warning("[x264] Cannot get next image\n");
+            flush=true;
+        }
+    }
+
+    nbNal = 0;
+    x264_picture_init(&pic_out);
+    out->flags = 0;
+
+    if(flush)
+    {
+        ADM_info("Flushing delayed frames\n");
+        er=x264_encoder_encode (handle, &nal, &nbNal, NULL, &pic_out);
+        if(er<0)
+        {
+            ADM_error ("[x264] Encode error %d while flushing delayed frames.\n",er);
+            return false;
+        }
+        if(!er && !x264_encoder_delayed_frames(handle))
+        {
+            ADM_info ("End of flush\n");
+            return false;
+        }
     }else
     {
-        //printf("[PPPP] x264 Incoming : %" PRIu64"us \n",image->Pts);    
-        // 2-preamble   
-        if(false==preAmble(image))
+        er=x264_encoder_encode (handle, &nal, &nbNal, &pic, &pic_out);
+        if(er<0)
         {
-            ADM_warning("[x264] preAmble failed\n");
+            ADM_error ("[x264] Error encoding %d\n",er);
             return false;
         }
     }
-    //
-      x264_nal_t        *nal;
-      int               nbNal = 0;
-      x264_picture_t    pic_out;
-
-      out->flags = 0;
-      
-        int er;
-        if(false==gotFrame)     
-        {
-            ADM_info("Flushing delayed frames\n");
-            er=x264_encoder_encode (handle, &nal, &nbNal, NULL, &pic_out);
-            if(er<=0)
-            {
-                ADM_info ("End of flush\n");
-                return false;
-            }
-        }else 
-        {
-            er=x264_encoder_encode (handle, &nal, &nbNal, &pic, &pic_out);
-            if(er<0)
-            {
-              ADM_error ("[x264] Error encoding %d\n",er);
-              return false;
-            }
-        }
-        if(!nbNal)
-        {
-            ADM_info("[x264] Null frame\n");
-            goto again;
-        }
+    if(!nbNal)
+    {
+        ADM_info("[x264] Null frame\n");
+        goto again;
+    }
 
 
     // 3-encode
@@ -244,8 +253,8 @@ bool  x264Encoder::preAmble (ADMImage * in)
       pic.img.i_csp = X264_CSP_I420;
       pic.img.i_plane = 3;
       pic.img.plane[0] = YPLANE(in);
-      pic.img.plane[2] = UPLANE(in);
-      pic.img.plane[1] = VPLANE(in);
+      pic.img.plane[1] = UPLANE(in);
+      pic.img.plane[2] = VPLANE(in);
       pic.img.i_stride[0] = in->GetPitch(PLANAR_Y);
       pic.img.i_stride[1] = in->GetPitch(PLANAR_U);
       pic.img.i_stride[2] = in->GetPitch(PLANAR_V);
@@ -269,6 +278,16 @@ bool x264Encoder::postAmble (ADMBitstream * out,uint32_t nbNals,x264_nal_t *nal,
         out->len=size;
         aprintf("--PostAmble--\n");
         // Make sure PTS & DTS > 0
+        if(firstIdr)
+        {
+            if(picout->i_dts<0)
+                encoderDelay=-picout->i_dts;
+            else
+                encoderDelay=0;
+            ADM_info("First IDR out of encoder with DTS = %" PRId64" us, setting encoder delay to %" PRId64" us.\n",
+                picout->i_dts,
+                (int64_t)encoderDelay);
+        }
         int64_t finalDts=picout->i_dts+(int64_t)getEncoderDelay();
         if(finalDts<0)
         {
@@ -311,7 +330,6 @@ bool x264Encoder::postAmble (ADMBitstream * out,uint32_t nbNals,x264_nal_t *nal,
           {
               // Put our SEI front...
               // first a temp location...
-              firstIdr=false;
               uint8_t *tmpBuffer=new uint8_t[size];
               memcpy(tmpBuffer,out->data,size);
               uint8_t *dout=out->data;
@@ -326,6 +344,7 @@ bool x264Encoder::postAmble (ADMBitstream * out,uint32_t nbNals,x264_nal_t *nal,
               out->len = size; // update total size
               delete [] tmpBuffer;
           }
+          firstIdr=false;
           break;
         case X264_TYPE_I:
           out->flags = AVI_P_FRAME;
